@@ -1,8 +1,7 @@
 package Crypt::SecretBuffer::INI;
 # VERSION
 # ABSTRACT: Parse INI format from a SecretBuffer
-$Crypt::SecretBuffer::INI::VERSION = '0.017_002'; # TRIAL
-$Crypt::SecretBuffer::INI::VERSION = '0.017002';
+$Crypt::SecretBuffer::INI::VERSION = '0.008';
 use strict;
 use warnings;
 use Carp;
@@ -197,7 +196,7 @@ sub parse {
       croak $tokens->{error}
          if defined $tokens->{error};
       if (defined $tokens->{section}) {
-         $tokens->{section}->copy_to($section= '');
+         $tokens->{section}->copy_to($section);
          if (defined $sep) {
             $node= $root;
             for (split $sep, $section) {
@@ -217,7 +216,7 @@ sub parse {
                push @$root, '', ($node= {});
             }
          }
-         $tokens->{key}->copy_to($key= '');
+         $tokens->{key}->copy_to($key);
          if (!$tokens->{value}) {
             $value= undef;
          } else {
@@ -349,30 +348,27 @@ which also means your values can't contain the comment character.
 This is an arrayref of rules that describe which flags should be applied to which
 type of keys.  Each element is of the form:
 
-  { key => $literal_or_regex, encoding => $enc, secret => $bool },
+  { key => $literal_or_pattern, flags => $flags },
 
 or
 
-  { section => $literal_or_regex, rules => [
+  { section => $header_or_regex, rules => [
      ...
   ]}
 
 As a convenience, that structure can be built from a shorter notation:
 
-  [
-    $literal_key     => \%attrs,
-    qr/$key_pattern/ => \%attrs,
-    $section_header => [
-      ...
-    ],
-    qr/$section_pattern/ => [
-      ...
-    ],
+  $literal_key     => $flags,
+  qr/$key_pattern/ => $flags,
+  $section_header => [
     ...
-  ]
+  ],
+  qr/$section_pattern/ => [
+    ...
+  ],
+  ...
 
-Note that the rule lists are arrays, not hashrefs.  THis allows them to have regexes as keys,
-and preserves order.  During a parse, rules are checked first to last, and the first match wins.
+During a parse, rules are processed in order, and the first match wins.
 
 =head1 METHODS
 
@@ -406,7 +402,7 @@ This function dies on any parse errors.
 
 =head1 VERSION
 
-version 0.017_002
+version 0.008
 
 =head1 AUTHOR
 
@@ -414,7 +410,7 @@ Michael Conrad <mike@nrdvana.net>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2026 by Michael Conrad.
+This software is copyright (c) 2025 by Michael Conrad.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
